#include "camera.h"
#include "conf.h"
#include "input.h"
#include "level.h"
#include "particles.h"
#include "player.h"
#include "stars.h"
#include "vec.h"
#include <gint/cpu.h>
#include <gint/display.h>
#include <gint/timer.h>
#include <stdlib.h>

static void update(void);
static void draw(void);
static int callback(volatile int *);

int
main(void)
{
	int timer;
	int frameskip = 1;
	volatile int has_ticked = 0;

	srand(1337);
	timer = timer_configure(TIMER_ANY, 1000000 / TARGET_FPS,
	                        GINT_CALL(callback, &has_ticked));
	timer_start(timer);
	input_init();
	stars_init();
	level_load(0);
	camera_init(player_pos());

	while (!input_down(K_EXIT)) {
		int i;
		draw();
		for (i = 0; i < frameskip; i++) {
			if (has_ticked > frameskip) {
				frameskip = has_ticked;
			}
			while (!has_ticked)
				sleep();
			while (has_ticked)
				has_ticked = 0;
			update();
		}
	}

	timer_stop(timer);
	level_free();
	return 1;
}

static void
update(void)
{
	particles_update();
	input_update();
	player_update();
	camera_update();
	stars_update();
}

static void
draw(void)
{
	dclear(C_BLACK);
	stars_draw();
	level_draw();
	player_draw();
	particles_draw();
	dupdate();
}

static int
callback(volatile int *arg)
{
	*arg += 1;
	return 0;
}
